﻿(function ($) {
    $.fn.extend({
        pageTable: function (options) {
            var defaults = {
                hasNo: true,//是否需要序号
                pageMode: 0,//分页模式，0为标准版,1为简化版，多用于APP，
                pageSize: 20,//每页数量
                data:null,
                columns: [],
                getDataFun: null,//获取数据的方法,通过这个方法，获取到指定数据，需传递如下参数,pageIndex,pageSize,和成功取到数据后的委托方法
                getParamFun: null,//获取参数的方法，用于ajax传值时使用
            }
            //分页区主要对象变量
            var spanTotal, pageNumber, spanPageSize, spanGO;
            var pageNumRequire;
            var btnPrev, btnNext;
            var allowPage = true;//是否需要分页

            var op = $.extend(defaults, options);

            allowPage = op.pageSize > 0 //表示要分页

            var totalPage = 0;//当前总页数
            var pageIndex = 0;//当前页的索引
            var totalRecord = 0
            var table = $(this);

            var thead = table.find("thead");
            if (thead == null || thead.length == 0) {
                thead = $("<thead></thead>");
                table.append(thead);
            }

            var tbody = table.find("tbody");
            if (tbody == null || tbody.length == 0) {
                tbody = $("<tbody></tbody>");
                table.append(tbody);
            }
            var tfoot = table.find("tfoot");
            if (tfoot == null || tfoot.length == 0) {
                tfoot = $("<tfoot></tfoot>");
                table.append(tfoot);
            }

            //创建列头
            createHeader();

            tbody.empty();

            //创建分页区
            tfoot.empty();
            createTFootTr();
            if (allowPage)
                tfoot.show();
            else
                tfoot.hide();

            //默认加载第一页
            if (op.data != null) {
                allowPage = false;
                createBody(op.data);
            }
            else
                gotoPage(0);

            //创建列头
            function createHeader() {
                thead.empty();
                var tr = $("<tr></tr>");
                thead.append(tr);

                if (op.hasNo == true)//有序号
                {
                    var td = $("<th>序号</th>");
                    tr.append(td);
                }

                $(op.columns).each(function (i, column) {
                    var td = $("<th></th>");
                    td.html(column.text);
                    tr.append(td);
                });
            }

            function createBody(rows) {
                tbody.empty();
                var rowNo = 0;
                $(rows).each(function (i, row) {
                    rowNo++;
                    var tr = $("<tr></tr>");
                    tbody.append(tr);

                    if (op.hasNo == true)//有序号
                    {
                        var td = $("<td>" + rowNo + "</td>");
                        tr.append(td);
                    }

                    $(op.columns).each(function (j, column) {
                        var td = $("<td></td>");
                        tr.append(td);
                        if (column.formatFun != null) {
                            var html = column.formatFun(row);
                            if (html == null || html == "null" || html == "undefined") html = "";
                            td.html(html);
                        }
                        else {
                            var v = row[column.field];
                            if (isEmptyValue(v)) v = "";
                            td.html(v);
                        }
                    });
                });
            }

            function isEmptyValue(v) {
                if (v == "0001-01-01 00:00:00")
                    return true;
                else if (v == null)
                    return true;
                else if (v == "null")
                    return true;
                else if (v == "00000000-0000-0000-0000-000000000000")
                    return true;
                else
                    return false;
            }
            //刷新并创建分页对象
            function createTFootTr() {
                tfoot.empty();
                var tr = $("<tr></tr>");
                tfoot.append(tr);

                var colspan = op.columns.length;
                if (op.hasNo) colspan++;


                var td = $("<td align=\"right\" colspan=\"" + colspan + "\"></td>");
                tr.append(td);

                var divPage = $("<div class=\"page tr\"></div>");
                td.append(divPage);

                spanTotal = $("<span></span>");//总数量区
                spanTotal.html("共" + totalRecord + "条&nbsp;");
                divPage.append(spanTotal);

                btnPrev = $("<a class=\"prevf\" href=\"javascript:void(0)\">上一页</a>");
                btnPrev.click(function () {
                    if (pageIndex > 0)
                        gotoPage(pageIndex - 1);
                });




                divPage.append(btnPrev);

                pageNumber = $("<span></span>");//各页导航按钮区，主要用于标准版
                //创建各页按钮区

                createPageBtn(pageIndex + 1, totalPage);
                if (op.pageMode == 0) //标准区
                    divPage.append(pageNumber);

                btnNext = $("<a class=\"nextf\" href=\"javascript:void(0)\">下一页</a>");
                btnNext.click(function () {
                    if (pageIndex < totalPage - 1) //不是最后一页
                        gotoPage(pageIndex + 1);
                });
                divPage.append(btnNext);


                spanPageSize = $("<span>共" + totalPage + "页</span>");//总页数区
                divPage.append(spanPageSize);


                spanGO = $("<span></span>");//跳转区
                pageNumRequire = $("<select></select>");
                spanGO.append("&nbsp;&nbsp;&nbsp;到");
                spanGO.append(pageNumRequire);
                spanGO.append("页");
                divPage.append(spanGO);

                for (var i = 0; i <= totalPage - 1; i++) {
                    var pageNo = i + 1;
                    var opItem = $("<option value='" + i + "'>" + pageNo + "</option>");
                    if (pageIndex == i)
                        opItem.attr("selected", "selected");
                    pageNumRequire.append(opItem);
                }
                pageNumRequire.change(function () {
                    var index = parseInt($(this).val());
                    if (isNaN(index)) index = 0;

                    gotoPage(index);
                });
            }

            //刷新页脚区
            function refreshTFoot() {
                spanTotal.html("共" + totalRecord + "条&nbsp;");
                if (pageIndex <= 0) //已是第一页
                    btnPrev.attr("disabled", "disabled");
                else
                    btnPrev.removeAttr("disabled");

                if (pageIndex > totalPage - 1)
                    btnNext.attr("disabled", "disabled");
                else
                    btnNext.removeAttr("disabled");


                //创建各页按钮区
                createPageBtn(pageIndex + 1, totalPage);


                spanPageSize.html("共" + totalPage + "页");//总页数区


                pageNumRequire.empty();
                for (var i = 0; i <= totalPage - 1; i++) {
                    var pageNo = i + 1;
                    var opItem = $("<option value='" + i + "'>" + pageNo + "</option>");
                    if (pageIndex == i)
                        opItem.attr("selected", "selected");
                    pageNumRequire.append(opItem);
                }
            }

            function createPageBtn(curPage, totalPage) {
                pageNumber.empty();

                if (totalPage <= 0)//少于
                    return;

                var arrNum = [];
                var spanIndex = -1;
                if (totalPage < 7) //当总的页数在6页之内时
                {
                    for (var i = 1; i <= totalPage; i++) {
                        arrNum.push(i);
                    }
                }
                else if (curPage < 4) //当前页少于4，则当前页是在前部
                {
                    arrNum = [1, 2, 3, 4, totalPage - 1, totalPage];
                    spanIndex = 3;
                }
                else if (totalPage - curPage < 2) {
                    arrNum = [1, 2, 3, totalPage - 2, totalPage - 1, totalPage];
                    spanIndex = 2;
                }
                else if (totalPage - curPage < 3) {
                    arrNum = [1, 2, totalPage - 3, totalPage - 2, totalPage - 1, totalPage];
                    spanIndex = 1;
                }
                else if (totalPage - curPage < 4) {
                    arrNum = [1, totalPage - 4, totalPage - 3, totalPage - 2, totalPage - 1, totalPage];
                    spanIndex = 0;
                }
                else {
                    arrNum = [curPage - 2, curPage - 1, curPage, curPage + 1, totalPage - 1, totalPage];
                    spanIndex = 3;
                }

                for (var i = 0; i < arrNum.length; i++) {
                    var curNo = arrNum[i] - 1;
                    var btn = $("<a href='javascript:;'></a>");
                    if (curPage == arrNum[i])
                        btn.attr("class", "acted");
                    else {
                        btn.data("pageIndex", curNo);
                        btn.click(function () {
                            var index = $(this).data("pageIndex");
                            gotoPage(index);
                        });
                    }

                    btn.html(arrNum[i]);
                    pageNumber.append(btn);

                    if (i == spanIndex) {
                        pageNumber.append("...&nbsp;");
                    }
                }
            }

           
            //取到数据后执行的方法
            function loadedData(data) {
                if (((data.total != null && data.total != undefined) && data.total <= 0) || data.rows == null || data.rows.length <= 0) {//没有数据
                    tbody.empty();
                    totalPage = 0;
                    pageIndex = 0;
                    totalRecord = 0;
                    refreshTFoot();

                    return;
                }
                var rows = [];
                if (data.total != null && data.rows != null)//标准格式返回的值
                {
                    //算出总页数
                    totalRecord = data.total;//总记录数
                    if (isNaN(totalRecord)) totalRecord = data.rows.length;
                    if (isNaN(totalRecord)) totalRecord = 0;

                    totalPage = Math.ceil(totalRecord / op.pageSize);

                    if (pageIndex > totalPage - 1) //比最后一页还大，则视为最后一页
                        pageIndex = totalPage - 1;

                    if (pageIndex < 0) pageIndex = 0;

                    refreshTFoot();

                    rows = data.rows;
                }
                else {
                    totalRecord = data.length;
                    totalPage = 1;
                    pageIndex = 0;
                    rows = data;
                }
                createBody(rows);
            }
            //转到第几页并显示指定页数据
            function gotoPage(index) {
                var successFun = function (data) {
                    pageIndex = index;
                    loadedData(data);
                };

                pageIndex = index;//设置当前页索引为该页

                if (op.getDataFun != null)
                    op.getDataFun(pageIndex, op.pageSize, successFun);
            }

            return {
                goto: gotoPage
            }
        }
    });
})($);