﻿//采用懒加载方式分页显示
(function ($) {
    $.fn.extend({
        WinTransitions: function () {
            var main = $(this);//当前窗体主体
            var pageWins = {};//存放当前已打开窗体对象

            var isAnimating = false;
            var endCurrPage = false;
            var endNextPage = false;
            var animEndEventNames = {
                'WebkitAnimation': 'webkitAnimationEnd',
                'OAnimation': 'oAnimationEnd',
                'msAnimation': 'MSAnimationEnd',
                'animation': 'animationend'
            };
            // animation end event name
            var animEndEventName = animEndEventNames[Modernizr.prefixed('animation')];
            // support css animations
            var support = Modernizr.cssanimations;

            var currentPage = null;//当前对象，下一对象


            //创建新的窗体
            function createNewWin(name, url, anmi) {
                //动画仅限如下，左右切换，上下切换，淡入淡出
                var pageWin = $("<div class=\"pt-page\" style=\"-webkit-overflow-scrolling:touch;\"></div>");
                var frame = $("<iframe style=\"width: 100% !important; height: 100% !important; border: 0px; overflow:auto\"></iframe>");

                pageWin.data('originalClassList', pageWin.attr('class'));
                pageWin.append(frame);

                main.append(pageWin);
                var style = getAnimStyle(anmi);

                return {
                    url: url,
                    anmi: anmi,
                    win: pageWin,
                    frame: frame,
                    inClass: inClass,
                    outClass: outClass,
                    prevPage: null //打开当前窗口的前一窗口
                };
            }

            //查到已存在的窗体
            function findWin(name) {
                var frm = pageWins[name];
                return frm;
            }

            function removeWin(page)
            {
                var newObj = {};
                for(var k in pageWins)
                {
                    if (page!=pageWins[k])
                    {
                        newObj[k] = pageWins[k];
                    }
                }
                pageWins = newObj;
            }

            function getAnimStyle(animation) {
                switch (animation) {

                    case 1://右入左出
                        outClass = 'pt-page-moveToLeft';
                        inClass = 'pt-page-moveFromRight';
                        break;
                    case 2://
                        outClass = 'pt-page-moveToRight';
                        inClass = 'pt-page-moveFromLeft';
                        break;
                    case 3:
                        outClass = 'pt-page-moveToTop';
                        inClass = 'pt-page-moveFromBottom';
                        break;
                    case 4:
                        outClass = 'pt-page-moveToBottom';
                        inClass = 'pt-page-moveFromTop';
                        break;
                    case 5:
                        outClass = 'pt-page-fade';
                        inClass = 'pt-page-moveFromRight pt-page-ontop';
                        break;
                    case 6:
                        outClass = 'pt-page-fade';
                        inClass = 'pt-page-moveFromLeft pt-page-ontop';
                        break;
                    case 7:
                        outClass = 'pt-page-fade';
                        inClass = 'pt-page-moveFromBottom pt-page-ontop';
                        break;
                    case 8:
                        outClass = 'pt-page-fade';
                        inClass = 'pt-page-moveFromTop pt-page-ontop';
                        break;
                    case 9:
                        outClass = 'pt-page-moveToLeftFade';
                        inClass = 'pt-page-moveFromRightFade';
                        break;
                    case 10:
                        outClass = 'pt-page-moveToRightFade';
                        inClass = 'pt-page-moveFromLeftFade';
                        break;
                    default:
                        outClass = 'pt-page-moveToLeft';
                        inClass = 'pt-page-moveFromRight';
                }
                return { inClass: inClass, outClass: outClass };
            }

            function openWin(name, url, animation, reftag) {
                //当前还在进行窗口切换，则不操作
                if (isAnimating) {
                    return false;
                }
                isAnimating = true;

                var page = findWin(name);//检测是否已存在，如果已存在，则打开该窗口
                if (page == null)//否则，则新添加窗口
                    page = createNewWin(name, url, animation);
                else
                    page.url = url;

                page.prevPage = currentPage;

                var $currPage = currentPage;//有可能是空值

                var $nextPage = page;//当前要打开的窗口

                outClass = page.outClass;
                inClass = page.inClass;

                //开始进行动画
                function beginAnim() {
                    $nextPage.win.addClass('pt-page-current');
                    if ($currPage != null) {
                        $currPage.win.addClass(outClass).on(animEndEventName, function () {
                            $currPage.win.off(animEndEventName);
                            endCurrPage = true;
                            if (endNextPage) {
                                onEndAnimation($currPage, $nextPage);
                            }
                        });
                    }

                    $nextPage.win.addClass(inClass).on(animEndEventName, function () {
                        $nextPage.win.off(animEndEventName);
                        endNextPage = true;
                        if ($currPage == null) endCurrPage = true;
                        if (endCurrPage) {
                            onEndAnimation($currPage, $nextPage);
                        }
                    });

                    if (!support) {
                        onEndAnimation($currPage, $nextPage);
                    }
                }
                $nextPage.frame.attr("src", $nextPage.url);
                beginAnim();               

                currentPage = page;//当前新开的窗口，便是当前窗口了
            }

            //关闭当前窗口
            function closeWin() {

                //当前还在进行窗口切换，则不操作
                if (isAnimating) {
                    return false;
                }
                isAnimating = true;

                var page = currentPage;
                if (page == null) return; //并无当前窗口

                var $currPage = currentPage;//有可能是空值

                var $nextPage = page.prevPage;
                var isNew = false;
                if ($nextPage == null)//没有上一窗口，则默认为首页
                {
                    $nextPage = createNewWin("home", "Home/Index", page.anmi);
                    isNew = true;
                }

                


                outClass = page.outClass;
                inClass = page.prevPage == null ? page.inClass : page.prevPage.inClass;

                //开始进行动画
                function beginAnim() {
                    $nextPage.win.addClass('pt-page-current');

                    $currPage.win.addClass(outClass).on(animEndEventName, function () {
                        $currPage.win.off(animEndEventName);
                        endCurrPage = true;
                        if (endNextPage) {
                            onCloseAnimation($currPage, $nextPage);
                        }
                    });

                    $nextPage.win.addClass(inClass).on(animEndEventName, function () {
                        $nextPage.win.off(animEndEventName);
                        endNextPage = true;
                        if (endCurrPage) {
                            onCloseAnimation($currPage, $nextPage);
                        }
                    });

                    if (!support) {
                        onCloseAnimation($currPage, $nextPage);
                    }
                }
                if (isNew) {
                    $nextPage.frame.attr("src", url);
                    beginAnim();
                }
                else {
                    beginAnim();
                }
                currentPage = page.prevPage;
            }

            function onEndAnimation($outpage, $inpage) {
                endCurrPage = false;
                endNextPage = false;
                resetPage($outpage, $inpage);
                isAnimating = false;
            }

            function onCloseAnimation($outpage, $inpage) {
                endCurrPage = false;
                endNextPage = false;
                try{
                    if ($outpage != null) {
                        $outpage.win.attr('class', $outpage.win.data('originalClassList'));
                        removeWin($outpage);
                        $outpage.win.remove();
                    }
                    if ($inpage != null) $inpage.win.attr('class', $inpage.win.data('originalClassList') + ' pt-page-current');
                }
                catch(e){}
                isAnimating = false;
            }

            function resetPage($outpage, $inpage) {
                if ($outpage != null) $outpage.win.attr('class', $outpage.win.data('originalClassList'));
                if ($inpage != null) $inpage.win.attr('class', $inpage.win.data('originalClassList') + ' pt-page-current');
            }

            return {
                openWin: openWin,
                closeWin:closeWin
            }
        }
    });
})($);


//已弃用
function PageTransitions(url) {
    var $main = $('#pt-main'),
		$pages = $main.children('div.pt-page'),
		animcursor = 1,
		pagesCount = $pages.length,
		current = 0,
		isAnimating = false,
		endCurrPage = false,
		endNextPage = false,
		animEndEventNames = {
		    'WebkitAnimation': 'webkitAnimationEnd',
		    'OAnimation': 'oAnimationEnd',
		    'msAnimation': 'MSAnimationEnd',
		    'animation': 'animationend'
		},
		// animation end event name
		animEndEventName = animEndEventNames[Modernizr.prefixed('animation')],
		// support css animations
		support = Modernizr.cssanimations;


    function init(myurl) {

        $pages.each(function () {
            var $page = $(this);
            $page.data('originalClassList', $page.attr('class'));
        });

        $pages.eq(current).addClass('pt-page-current').find("iframe").attr("src", myurl);

    }

    function nextPage(animation, url) {

        if (isAnimating) {
            return false;
        }

        isAnimating = true;

        var $currPage = $pages.eq(current);

        if (current < pagesCount - 1) {
            ++current;
        }
        else {
            current = 0;
        }

        var $nextPage = $pages.eq(current),
			outClass = '', inClass = '';

        $nextPage.find("iframe").unbind();


        var outInClass = getAnimStyle(animation)

        outClass = outInClass.outClass;
        inClass = outInClass.inClass;


        function getAnimStyle(animation) {
            var outClass = '', inClass = '';
            switch (animation) {

                case 1://右入左出
                    outClass = 'pt-page-moveToLeft';
                    inClass = 'pt-page-moveFromRight';
                    break;
                case 2://
                    outClass = 'pt-page-moveToRight';
                    inClass = 'pt-page-moveFromLeft';
                    break;
                case 3:
                    outClass = 'pt-page-moveToTop';
                    inClass = 'pt-page-moveFromBottom';
                    break;
                case 4:
                    outClass = 'pt-page-moveToBottom';
                    inClass = 'pt-page-moveFromTop';
                    break;
                case 5:
                    outClass = 'pt-page-fade';
                    inClass = 'pt-page-moveFromRight pt-page-ontop';
                    break;
                case 6:
                    outClass = 'pt-page-fade';
                    inClass = 'pt-page-moveFromLeft pt-page-ontop';
                    break;
                case 7:
                    outClass = 'pt-page-fade';
                    inClass = 'pt-page-moveFromBottom pt-page-ontop';
                    break;
                case 8:
                    outClass = 'pt-page-fade';
                    inClass = 'pt-page-moveFromTop pt-page-ontop';
                    break;
                case 9:
                    outClass = 'pt-page-moveToLeftFade';
                    inClass = 'pt-page-moveFromRightFade';
                    break;
                case 10:
                    outClass = 'pt-page-moveToRightFade';
                    inClass = 'pt-page-moveFromLeftFade';
                    break;
                case 11:
                    outClass = 'pt-page-moveToTopFade';
                    inClass = 'pt-page-moveFromBottomFade';
                    break;
                case 12:
                    outClass = 'pt-page-moveToBottomFade';
                    inClass = 'pt-page-moveFromTopFade';
                    break;
                case 13:
                    outClass = 'pt-page-moveToLeftEasing pt-page-ontop';
                    inClass = 'pt-page-moveFromRight';
                    break;
                case 14:
                    outClass = 'pt-page-moveToRightEasing pt-page-ontop';
                    inClass = 'pt-page-moveFromLeft';
                    break;
                case 15:
                    outClass = 'pt-page-moveToTopEasing pt-page-ontop';
                    inClass = 'pt-page-moveFromBottom';
                    break;
                case 16:
                    outClass = 'pt-page-moveToBottomEasing pt-page-ontop';
                    inClass = 'pt-page-moveFromTop';
                    break;
                case 17:
                    outClass = 'pt-page-scaleDown';
                    inClass = 'pt-page-moveFromRight pt-page-ontop';
                    break;
                case 18:
                    outClass = 'pt-page-scaleDown';
                    inClass = 'pt-page-moveFromLeft pt-page-ontop';
                    break;
                case 19:
                    outClass = 'pt-page-scaleDown';
                    inClass = 'pt-page-moveFromBottom pt-page-ontop';
                    break;
                case 20:
                    outClass = 'pt-page-scaleDown';
                    inClass = 'pt-page-moveFromTop pt-page-ontop';
                    break;
                case 21:
                    outClass = 'pt-page-scaleDown';
                    inClass = 'pt-page-scaleUpDown pt-page-delay300';
                    break;
                case 22:
                    outClass = 'pt-page-scaleDownUp';
                    inClass = 'pt-page-scaleUp pt-page-delay300';
                    break;
                case 23:
                    outClass = 'pt-page-moveToLeft pt-page-ontop';
                    inClass = 'pt-page-scaleUp';
                    break;
                case 24:
                    outClass = 'pt-page-moveToRight pt-page-ontop';
                    inClass = 'pt-page-scaleUp';
                    break;
                case 25:
                    outClass = 'pt-page-moveToTop pt-page-ontop';
                    inClass = 'pt-page-scaleUp';
                    break;
                case 26:
                    outClass = 'pt-page-moveToBottom pt-page-ontop';
                    inClass = 'pt-page-scaleUp';
                    break;
                case 27:
                    outClass = 'pt-page-scaleDownCenter';
                    inClass = 'pt-page-scaleUpCenter pt-page-delay400';
                    break;
                case 28:
                    outClass = 'pt-page-rotateRightSideFirst';
                    inClass = 'pt-page-moveFromRight pt-page-delay200 pt-page-ontop';
                    break;
                case 29:
                    outClass = 'pt-page-rotateLeftSideFirst';
                    inClass = 'pt-page-moveFromLeft pt-page-delay200 pt-page-ontop';
                    break;
                case 30:
                    outClass = 'pt-page-rotateTopSideFirst';
                    inClass = 'pt-page-moveFromTop pt-page-delay200 pt-page-ontop';
                    break;
                case 31:
                    outClass = 'pt-page-rotateBottomSideFirst';
                    inClass = 'pt-page-moveFromBottom pt-page-delay200 pt-page-ontop';
                    break;
                case 32:
                    outClass = 'pt-page-flipOutRight';
                    inClass = 'pt-page-flipInLeft pt-page-delay500';
                    break;
                case 33:
                    outClass = 'pt-page-flipOutLeft';
                    inClass = 'pt-page-flipInRight pt-page-delay500';
                    break;
                case 34:
                    outClass = 'pt-page-flipOutTop';
                    inClass = 'pt-page-flipInBottom pt-page-delay500';
                    break;
                case 35:
                    outClass = 'pt-page-flipOutBottom';
                    inClass = 'pt-page-flipInTop pt-page-delay500';
                    break;
                case 36:
                    outClass = 'pt-page-rotateFall pt-page-ontop';
                    inClass = 'pt-page-scaleUp';
                    break;
                case 37:
                    outClass = 'pt-page-rotateOutNewspaper';
                    inClass = 'pt-page-rotateInNewspaper pt-page-delay500';
                    break;
                case 38:
                    outClass = 'pt-page-rotatePushLeft';
                    inClass = 'pt-page-moveFromRight';
                    break;
                case 39:
                    outClass = 'pt-page-rotatePushRight';
                    inClass = 'pt-page-moveFromLeft';
                    break;
                case 40:
                    outClass = 'pt-page-rotatePushTop';
                    inClass = 'pt-page-moveFromBottom';
                    break;
                case 41:
                    outClass = 'pt-page-rotatePushBottom';
                    inClass = 'pt-page-moveFromTop';
                    break;
                case 42:
                    outClass = 'pt-page-rotatePushLeft';
                    inClass = 'pt-page-rotatePullRight pt-page-delay180';
                    break;
                case 43:
                    outClass = 'pt-page-rotatePushRight';
                    inClass = 'pt-page-rotatePullLeft pt-page-delay180';
                    break;
                case 44:
                    outClass = 'pt-page-rotatePushTop';
                    inClass = 'pt-page-rotatePullBottom pt-page-delay180';
                    break;
                case 45:
                    outClass = 'pt-page-rotatePushBottom';
                    inClass = 'pt-page-rotatePullTop pt-page-delay180';
                    break;
                case 46:
                    outClass = 'pt-page-rotateFoldLeft';
                    inClass = 'pt-page-moveFromRightFade';
                    break;
                case 47:
                    outClass = 'pt-page-rotateFoldRight';
                    inClass = 'pt-page-moveFromLeftFade';
                    break;
                case 48:
                    outClass = 'pt-page-rotateFoldTop';
                    inClass = 'pt-page-moveFromBottomFade';
                    break;
                case 49:
                    outClass = 'pt-page-rotateFoldBottom';
                    inClass = 'pt-page-moveFromTopFade';
                    break;
                case 50:
                    outClass = 'pt-page-moveToRightFade';
                    inClass = 'pt-page-rotateUnfoldLeft';
                    break;
                case 51:
                    outClass = 'pt-page-moveToLeftFade';
                    inClass = 'pt-page-rotateUnfoldRight';
                    break;
                case 52:
                    outClass = 'pt-page-moveToBottomFade';
                    inClass = 'pt-page-rotateUnfoldTop';
                    break;
                case 53:
                    outClass = 'pt-page-moveToTopFade';
                    inClass = 'pt-page-rotateUnfoldBottom';
                    break;
                case 54:
                    outClass = 'pt-page-rotateRoomLeftOut pt-page-ontop';
                    inClass = 'pt-page-rotateRoomLeftIn';
                    break;
                case 55:
                    outClass = 'pt-page-rotateRoomRightOut pt-page-ontop';
                    inClass = 'pt-page-rotateRoomRightIn';
                    break;
                case 56:
                    outClass = 'pt-page-rotateRoomTopOut pt-page-ontop';
                    inClass = 'pt-page-rotateRoomTopIn';
                    break;
                case 57:
                    outClass = 'pt-page-rotateRoomBottomOut pt-page-ontop';
                    inClass = 'pt-page-rotateRoomBottomIn';
                    break;
                case 58:
                    outClass = 'pt-page-rotateCubeLeftOut pt-page-ontop';
                    inClass = 'pt-page-rotateCubeLeftIn';
                    break;
                case 59:
                    outClass = 'pt-page-rotateCubeRightOut pt-page-ontop';
                    inClass = 'pt-page-rotateCubeRightIn';
                    break;
                case 60:
                    outClass = 'pt-page-rotateCubeTopOut pt-page-ontop';
                    inClass = 'pt-page-rotateCubeTopIn';
                    break;
                case 61:
                    outClass = 'pt-page-rotateCubeBottomOut pt-page-ontop';
                    inClass = 'pt-page-rotateCubeBottomIn';
                    break;
                case 62:
                    outClass = 'pt-page-rotateCarouselLeftOut pt-page-ontop';
                    inClass = 'pt-page-rotateCarouselLeftIn';
                    break;
                case 63:
                    outClass = 'pt-page-rotateCarouselRightOut pt-page-ontop';
                    inClass = 'pt-page-rotateCarouselRightIn';
                    break;
                case 64:
                    outClass = 'pt-page-rotateCarouselTopOut pt-page-ontop';
                    inClass = 'pt-page-rotateCarouselTopIn';
                    break;
                case 65:
                    outClass = 'pt-page-rotateCarouselBottomOut pt-page-ontop';
                    inClass = 'pt-page-rotateCarouselBottomIn';
                    break;
                case 66:
                    outClass = 'pt-page-rotateSidesOut';
                    inClass = 'pt-page-rotateSidesIn pt-page-delay200';
                    break;
                case 67:
                    outClass = 'pt-page-rotateSlideOut';
                    inClass = 'pt-page-rotateSlideIn';
                    break;
            }
            return { outClass: outClass, inClass: inClass };
        }

        function beginAnim() {
            $nextPage.addClass('pt-page-current')
            $currPage.addClass(outClass).on(animEndEventName, function () {
                $currPage.off(animEndEventName);
                endCurrPage = true;
                if (endNextPage) {
                    onEndAnimation($currPage, $nextPage);
                }
            });

            $nextPage.addClass(inClass).on(animEndEventName, function () {
                $nextPage.off(animEndEventName);
                endNextPage = true;
                if (endCurrPage) {
                    onEndAnimation($currPage, $nextPage);
                }
            });

            if (!support) {
                onEndAnimation($currPage, $nextPage);
            }
        }



        $nextPage.find("iframe").attr("src", url);

        $nextPage.find("iframe").load(function () {
            beginAnim();
            $(this).unbind();
        });
    }

    function onEndAnimation($outpage, $inpage) {
        endCurrPage = false;
        endNextPage = false;
        resetPage($outpage, $inpage);
        isAnimating = false;
    }

    function resetPage($outpage, $inpage) {
        $outpage.attr('class', $outpage.data('originalClassList'));
        $inpage.attr('class', $inpage.data('originalClassList') + ' pt-page-current');
    }
    function toNext(url) {
        if (isAnimating) {
            return false;
        }
        if (animcursor > 67) {
            animcursor = 1;
        }
        nextPage(animcursor, url);
        ++animcursor;
    }
    init(url);
    return { init: init, toNext: toNext };

}


//区块之间切换
function PanelTransitions(options) {
    var animcursor = 1,
		current = 0,
		isAnimating = false,
		endCurrPage = false,
		endNextPage = false,
		animEndEventNames = {
		    'WebkitAnimation': 'webkitAnimationEnd',
		    'OAnimation': 'oAnimationEnd',
		    'msAnimation': 'MSAnimationEnd',
		    'animation': 'animationend'
		},
		// animation end event name
		animEndEventName = animEndEventNames[Modernizr.prefixed('animation')],
		// support css animations
		support = Modernizr.cssanimations;

    var defaults = {
        mainPanel: null, //主显示区
        pushPanel: null  //辅显示区
    };
    var op = $.extend(defaults, options);

    //未执行切换对象，则不执行
    if (op.pushPanel == null || op.pushPanel == null)
        return;

    op.mainPanel.data('originalClassList', op.mainPanel.attr('class'));
    op.pushPanel.data('originalClassList', op.pushPanel.attr('class'));

    //推送显示辅显示区
    function showType(isLF) {
        var df = isLF ? 1 : 3;

        nextPage(op.pushPanel, op.mainPanel, df);
    }

    //返回显示主区
    function showMain(isLF) {
        var df = isLF ? 2 : 4;
        nextPage(op.mainPanel, op.pushPanel, df);
    }

    function nextPage(showP, hideP, animation) {

        if (isAnimating) {
            return false;
        }

        isAnimating = true;

        var $currPage = hideP;

        var $nextPage = showP,
			outClass = '', inClass = '';

        showP.addClass('pt-page-current');


        var outInClass = getAnimStyle(animation)

        outClass = outInClass.outClass;
        inClass = outInClass.inClass;


        function getAnimStyle(animation) {
            var outClass = '', inClass = '';
            switch (animation) {

                case 1:
                    outClass = 'pt-page-moveToLeft';
                    inClass = 'pt-page-moveFromRight';
                    break;
                case 2:
                    outClass = 'pt-page-moveToRight';
                    inClass = 'pt-page-moveFromLeft';
                    break;
                case 3:
                    outClass = 'pt-page-moveToTop';
                    inClass = 'pt-page-moveFromBottom';
                    break;
                case 4:
                    outClass = 'pt-page-moveToBottom';
                    inClass = 'pt-page-moveFromTop';
                    break;
            }
            return { outClass: outClass, inClass: inClass };
        }

        $currPage.addClass(outClass).on(animEndEventName, function () {
            $currPage.off(animEndEventName);
            endCurrPage = true;
            if (endNextPage) {
                onEndAnimation($currPage, $nextPage);
            }
        });

        $nextPage.addClass(inClass).on(animEndEventName, function () {
            $nextPage.off(animEndEventName);
            endNextPage = true;
            if (endCurrPage) {
                onEndAnimation($currPage, $nextPage);
            }
        });

        if (!support) {
            onEndAnimation($currPage, $nextPage);
        }
    }

    function onEndAnimation($outpage, $inpage) {
        endCurrPage = false;
        endNextPage = false;
        resetPage($outpage, $inpage);
        isAnimating = false;
    }

    function resetPage($outpage, $inpage) {
        $outpage.attr('class', $outpage.data('originalClassList'));
        $inpage.attr('class', $inpage.data('originalClassList') + ' pt-page-current');
    }
    return { showType: showType, showMain: showMain };
}


//推送方式显示指定的URL
function PushUrl(options) {
    var animcursor = 1,
		current = 0,
		isAnimating = false,
		endCurrPage = false,
		endNextPage = false,
		animEndEventNames = {
		    'WebkitAnimation': 'webkitAnimationEnd',
		    'OAnimation': 'oAnimationEnd',
		    'msAnimation': 'MSAnimationEnd',
		    'animation': 'animationend'
		},
		// animation end event name
		animEndEventName = animEndEventNames[Modernizr.prefixed('animation')],
		// support css animations
		support = Modernizr.cssanimations;

    var defaults = {
        panel: null, //显示区对象
        url: "",
        showAnimation: 1//从左到右
    };
    var op = $.extend(defaults, options);

    //未执行切换对象，则不执行


    function show() {
        animation = op.showAnimation;

        if (isAnimating) {
            return false;
        }

        isAnimating = true;

        //   var $currPage = null;

        var $nextPage = op.panel,
			outClass = '', inClass = '';

        $nextPage.addClass('pt-page-current');


        var outInClass = getAnimStyle(animation)

        outClass = outInClass.outClass;
        inClass = outInClass.inClass;

        function getAnimStyle(animation) {
            var outClass = '', inClass = '';
            switch (animation) {

                case 1:
                    inClass = 'pt-page-moveFromRight';
                    break;
                case 2:
                    inClass = 'pt-page-moveFromLeft';
                    break;
                case 3:
                    inClass = 'pt-page-moveFromBottom';
                    break;
                case 4:
                    inClass = 'pt-page-moveFromTop';
                    break;
            }
            return { outClass: outClass, inClass: inClass };
        }

        //$currPage.addClass(outClass).on(animEndEventName, function () {
        //    $currPage.off(animEndEventName);
        //    endCurrPage = true;
        //    if (endNextPage) {
        //        onEndAnimation($currPage, $nextPage);
        //    }
        //});
        $("body").css("overflow", "hidden");
        $nextPage.addClass(inClass).on(animEndEventName, function () {
            $nextPage.off(animEndEventName);
            endNextPage = true;
            onEndAnimation(null, $nextPage);
        });

        if (!support) {
            onEndAnimation(null, $nextPage);
        }
    }

    function hide() {
        if (op.showAnimation == 1)
            animation = 2;
        else if (op.showAnimation == 2)
            animation = 1;
        else if (op.showAnimation == 3)
            animation = 4;
        else if (op.showAnimation == 4)
            animation = 3;
        else
            animation = 2;

        if (isAnimating) {
            return false;
        }

        isAnimating = true;

        var $currPage = op.panel;

        //  var $nextPage = showP;
        var outClass = '', inClass = '';

        //showP.addClass('pt-page-current');


        var outInClass = getAnimStyle(animation)

        outClass = outInClass.outClass;
        inClass = outInClass.inClass;

        function getAnimStyle(animation) {
            var outClass = '', inClass = '';
            switch (animation) {
                case 1:
                    outClass = 'pt-page-moveToLeft';
                    inClass = 'pt-page-moveFromRight';
                    break;
                case 2:
                    outClass = 'pt-page-moveToRight';
                    inClass = 'pt-page-moveFromLeft';
                    break;
                case 3:
                    outClass = 'pt-page-moveToTop';
                    inClass = 'pt-page-moveFromBottom';
                    break;
                case 4:
                    outClass = 'pt-page-moveToBottom';
                    inClass = 'pt-page-moveFromTop';
                    break;
            }
            return { outClass: outClass, inClass: inClass };
        }

        $currPage.addClass(outClass).on(animEndEventName, function () {
            $currPage.off(animEndEventName);
            endCurrPage = true;
            onEndAnimation($currPage, null);
            $("body").css("overflow", "auto");
        });

        if (!support) {
            onEndAnimation($currPage, null);
        }
    }


    function onEndAnimation($outpage, $inpage) {
        endCurrPage = false;
        endNextPage = false;
        resetPage($outpage, $inpage);
        isAnimating = false;
    }

    function resetPage($outpage, $inpage) {

        if ($outpage != null) $outpage.attr('class', $outpage.data('originalClassList'));
        if ($inpage != null) $inpage.attr('class', $inpage.data('originalClassList') + ' pt-page-current');
    }

    


    function init() {
        if (op.panel == null) {
            op.panel = $("<div class='pt-page'></div>");
            
            if (window.isiOS)
            {
                var h = "20px";
                if (isIPhoneX()) h = "40px";
                op.panel.css("padding-top", h);// "20px");
            }

            op.panel.css("z-index", "9999");
            $("body").append(op.panel);
        }

        if (op.url != "") {
            $.get(op.url, { stamp: Math.random() }, function (html, status) {
                op.panel.html(html);
            });
        }
        op.panel.data('originalClassList', op.panel.attr('class'));
    }

    init();

    return { show: show, hide: hide };
}


